#include <avr/io.h>
#include <avr/interrupt.h>

// ===== USER SETTINGS =====
#define NUMBER_MODES			5		//Count of modes/effects
// ------------------------------------
#define PWM_RESOLUTION			64		//Bit PWM (64 - 6-bit)  - 500Hz 
										///////// (127 - 7-bit) - 300Hz
										///////// (255 - 8-bit) - 130Hz
// ------------------------------------	
#define DELAY_BOUNCE_BUTTON		1480	//Button chatter elimination (~40ms) //(x - count) * 0.027ms
#define DELAY_SWITCH_BLINK		15000	//Garland Channel Switching	 (405ms) //---------------------
// ------------------------------------
#define DELAY_DIMMING_MODE_1	1000	//PWM values increase time, for the effect this value is half	  (1000ms) //(x - count) * 1ms 
#define DELAY_DIMMING_MODE_2	110		//PWM values increase time, for the effect this value is complete (110ms)  //-----------------

// ===== VARIABLES =====
// ----- Processing button -----
volatile int timerCounterButton = 0;
volatile char pressButton = 0;

// ----- Flashing effect -----
volatile char counterNumberChannel = 0;
volatile int delayCounterBlink = 0;

// ----- Pulse-Width Modulation -----
volatile char counterPWM = 0;
char channelPWM = 0;

// ----- Time delay for effects -----
volatile int timerCounterDelay = 0;
char timerCounterMilliseconds = 0;
volatile int delayMilliseconds = 0;

// ----- Flags for effects -----
char dimmingFlag = 1;
volatile char delayFlag = 0;

// ----- Counter of modes/effects -----
char counterMode = 0;

// ----- Array of meaning garland lines -----
char numberChannel[5] = {0x03, 0x06, 0x0A, 0x12, 0xFF};

ISR(TIM0_OVF_vect)
{
	//Interrupt Overflow Timer configured for every 0.027ms at frequency 9.6MHZ
	//It is configured for PWM speed
	
	// ----- Processing button -----
	if((PINB & 0x02) == 0){timerCounterButton++;}
	else{timerCounterButton = 0;}
	if(timerCounterButton > DELAY_BOUNCE_BUTTON){pressButton = 1;}
	
	// ----- Flashing effect -----
	if(delayCounterBlink <= DELAY_SWITCH_BLINK){delayCounterBlink++;}
	else{delayCounterBlink = 0; counterNumberChannel++;}
	if(counterNumberChannel == 4){counterNumberChannel = 0;}
	
	// ----- Time delay for effects -----
	if(timerCounterDelay <= delayMilliseconds){timerCounterDelay++;}
	else{timerCounterDelay = 0; delayFlag = 1;}
	
	// ----- PWM -----
	if(counterPWM <= PWM_RESOLUTION){counterPWM++;}
	else{counterPWM = 0;}
}

void initTimer()
{
	TCCR0B = 0x01;	//No prescaler
	TCNT0=0;		//Reset counter (0-255)
	TIMSK0 = 0x02;	//Enable interrupt overflow Timer0
}

void initPorts()
{
	DDRB = 0x1D;	//Output: PB0, PB2, PB3, PB4		Input: PB1
	PORTB = 0x02;	//Logic low: PB0, PB2, PB3, PB4		Pull-up: PB1
}

void init()
{
	initPorts();
	initTimer();
	
	sei();			//Enable global interrupt
}

char switchMode(char local_pressButton)
{
	//Function for processing a pressed button and a counting a mode number/effects

	// ----- Local variables -----
	char local_pressedButton = 0;
	
	// ----- Processing button -----
	if((local_pressButton == 1) && ((PINB & 0x02) != 0)){local_pressButton = 0; pressButton = 0; local_pressedButton = 1;}
	if(local_pressedButton == 1){local_pressedButton = 0; counterMode++;}
	if(counterMode > NUMBER_MODES){counterMode = 1;}
	
	// ----- Return -----
	return counterMode;
}

void onGarland()
{
	//Turn on all garland lines
	
	PORTB = 0xFF;
}

void offGarland()
{
	//Turn off all garland lines and pull-up button
	
	PORTB = 0x02;
}

void blinkGarland(unsigned char local_counterBlink)
{
	//Garland line switching
	
	PORTB = numberChannel[local_counterBlink];
}

void PWMGarland(char local_countChannelPWM, unsigned char local_channelPWM)
{
	//Function to output PWM to a unit channel
	
	if(counterPWM < local_countChannelPWM){PORTB = numberChannel[local_channelPWM];}
	else{PORTB = 0x02;}
}

void dimmingGarland(char local_countChannelPWM)
{
	// ----- Local variables -----
	unsigned char local_channelPWM = 4;
	delayMilliseconds = DELAY_DIMMING_MODE_1;
	
	//Gradual increase and decay
	if(delayFlag == 1)
	{
		delayFlag = 0;
		if(dimmingFlag == 1){timerCounterMilliseconds++;}
		if(timerCounterMilliseconds == PWM_RESOLUTION){dimmingFlag = 0;}
		if(dimmingFlag == 0){timerCounterMilliseconds--;}
		if(timerCounterMilliseconds == 0){dimmingFlag = 1;}
	}
	
	// ----- Using the PWM function -----
	PWMGarland(local_countChannelPWM, local_channelPWM);
}

void brightnessGarland(char local_countChannelPWM)
{	
	//A mode combining smooth on and off daisies and channel switching
	
	// ----- Local variables -----
	delayMilliseconds = DELAY_DIMMING_MODE_2;
	
	// ----- Gradual increase and decay -----
	if(delayFlag == 1)
	{
		delayFlag = 0;
		if(dimmingFlag == 1){timerCounterMilliseconds++;}
		if(timerCounterMilliseconds == PWM_RESOLUTION){dimmingFlag = 0;}
		if(dimmingFlag == 0){timerCounterMilliseconds--;}
		if(timerCounterMilliseconds == 0){dimmingFlag = 1; channelPWM++;}	//Channel switching at the end of the PWM cycle
		if(channelPWM == 4){channelPWM = 0;}
	}
	
	// ----- Using the PWM function -----
	PWMGarland(local_countChannelPWM, channelPWM);
}

int main(void)
{
    init();		//Initialization peripheral
    while(1)
    {
		// -----Local variables -----
		char mode = 0;
		
		// ----- Account mod number -----
		mode = switchMode(pressButton);
		
		// ----- List of all mode/effect -----
		if(mode == 1){onGarland();}
		if(mode == 2){blinkGarland(counterNumberChannel);}
		if(mode == 3){dimmingGarland(timerCounterMilliseconds);}
		if(mode == 4){brightnessGarland(timerCounterMilliseconds);}
		if(mode == 5){offGarland();}
	}
}

